local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- Implement a dummy TTD function if we do not get one from another module
if type(s.TimeToDie) ~= 'function'  then
	a.print(L["No TimeToDie function found, using a dummy function. Consider installing Pusikas' Spellflash: TTD."])
	s.TimeToDie = function(unit)
		return 10
	end
end

local regenRate
a.energyRegeneration = function()
	_, regenRate = GetPowerRegen()
	regenRate = format("%.2f", regenRate)
	if s.Buff(115288--[[Energizing Brew]], "player") then
		regenRate = regenRate + 10
	end
	return regenRate
end

local HasteCDs = {
	2825--[[Bloodlust]],
	26297--[[Berserking]],
	32182--[[Heroism]],
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	90355--[[Ancient Hysteria]],
	146555--[[Drums of Rage]],
}
a.Hasted = function()
	return s.Buff(HasteCDs, "player")
end

-- Detect raid-wide haste buffs, so we know when to
-- use potions
local BloodlustSpells = {
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	80353--[[Time Warp]],
	90355--[[Ancient Hysteria]],
	146555--[[Drums of Rage]],
}
a.Bloodlust = function()
	return s.Buff(BloodlustSpells, "player")
end

a.MySelf = UnitGUID("player")

a.IncomingDamage = {}
local LastAoETimestamp = 0
local CurrentTimestamp = 0
local LastAoETargetsHit = 0
local LastAoESpellID = 0
local event, sourceGUID, destGUID, spellId, spellName, amount
local DamagingAoESpells = {
	[101545--[[Flying Serpent Kick]]] = true,
	[107270--[[Spinning Crane Kick]]] = true,
	[113656--[[Fists of Fury]]] = true,
	[115181--[[Breath of Fire]]] = true,
	[148187--[[Rushing Jade Wind]]] = true,
}
local arrayIndex = 0
local tsIndex = 0
local myLastDamageTaken = time()

local function clear()
	for i = 1, 20, 1 do
		a.IncomingDamage[i] = 0
	end
end
clear()

a.updateAoE = function(spellId)
	if spellId ~= LastAoESpellID then
		LastAoETargetsHit = 1
		LastAoETimestamp = CurrentTimestamp
		LastAoESpellID = spellId
	elseif CurrentTimestamp == LastAoETimestamp then
		LastAoETargetsHit = LastAoETargetsHit + 1
	else
		LastAoETargetsHit = 1
		LastAoETimestamp = CurrentTimestamp
	end
end

a.COMBAT_LOG_EVENT_UNFILTERED = function (event, ...)
	CurrentTimestamp, event, _, sourceGUID, _, _, _, destGUID, _, _, _, spellId, spellName = ...
	tsIndex = mod(floor(CurrentTimestamp * 10), 20) + 1
	
	if tsIndex > arrayIndex then
	-- the new index comes after the old index. we have to clear all array entries between them.
		for i = arrayIndex + 1, tsIndex, 1 do
			a.IncomingDamage[i] = 0
		end
		arrayIndex = tsIndex
	elseif tsIndex < arrayIndex then
	-- the new index comes before the old index. we have to clear till the end of the array and then
	-- from the start up to the new index.
		for i = arrayIndex + 1, 20, 1 do
			a.IncomingDamage[i] = 0
		end
		for i = 1, tsIndex, 1 do
			a.IncomingDamage[i] = 0
		end
		arrayIndex = tsIndex
	end
	
	if sourceGUID == a.MySelf and destGUID ~= a.MySelf then
		if (event == "SPELL_DAMAGE" or event == "SPELL_MISSED") and DamagingAoESpells[spellId] then
			a.updateAoE(spellId)
		elseif (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and spellId == 115798--[[Weakened Blows]] then
			a.updateAoE(spellId)
		elseif (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and LastAoESpellID ~= 115798--[[Weakened Blows]] and spellId == 116330--[[Dizzying Haze]] then
			a.updateAoE(spellId)
		elseif (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and spellId == 130320--[[Rising Sun Kick]] then
			a.updateAoE(spellId)
		end
	elseif destGUID == a.MySelf and string.find(event, "_DAMAGE") then
		myLastDamageTaken = CurrentTimestamp
		if destGUID == a.MySelf and (event == "SWING_DAMAGE") then
			amount = select(12, ...)
			a.IncomingDamage[i] = a.IncomingDamage[i] + amount
		else
			amount = select(15, ...)
			a.IncomingDamage[i] = a.IncomingDamage[i] + amount
		end
	end
end
s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", a.COMBAT_LOG_EVENT_UNFILTERED)

-- Check if we are in AoE mode
a.ActiveEnemies = function()
	if CurrentTimestamp < LastAoETimestamp + 4 then
		return LastAoETargetsHit
	else
		return 1
	end
end

-- Scan for Readiness
x.CooldownReduction = 1
local AOC = {
	Trinket0Slot = {104974, 104725, 102292, 105223, 104476, 105472}, --[[Trinket 1]]
	Trinket1Slot = {104974, 104725, 102292, 105223, 104476, 105472}, --[[Trinket 2]]
}
local count, iLevel, temp
a.scanEquippedItems = function()
	count = 0
	for slot, list in pairs(AOC) do
		if s.Equipped(list, slot) then
			count = count + 1
			_, _, _, iLevel = GetItemInfo(GetInventoryItemLink("player", GetInventorySlotInfo(slot)))
			temp = 1 / (1 + 17 * math.pow(1.15, ((iLevel - 463) / 15)))
		end
	end
	if count >= 1 then
		if temp ~= x.CooldownReduction then
			x.CooldownReduction = temp
			a.print(L["Found Assurance of Consequence. Increased cooldown recovery rate calculated."])
		end
	else
		if x.CooldownReduction ~= 1 then
			x.CooldownReduction = 1
			a.print(L["No Assurance of Consequence found."])
		end
	end
end

s.RegisterModuleEvent(AddonName, "PLAYER_EQUIPMENT_CHANGED", function(event, slot) if slot == 13 or slot == 14 then a.scanEquippedItems() end end)
a.scanEquippedItems()

function a.getStagger()
	local name, val2
	name,_,_,_,_,_,_,_,_,_,_,_,_,_,_,val2 = UnitAura("player", s.SpellName(124275--[[Light Stagger]]), nil, "HARMFUL")
	if name then
		return val2
	end
	name,_,_,_,_,_,_,_,_,_,_,_,_,_,_,val2 = UnitAura("player", s.SpellName(124274--[[Moderate Stagger]]), nil, "HARMFUL")
	if name then
		return val2
	end
	name,_,_,_,_,_,_,_,_,_,_,_,_,_,_,val2 = UnitAura("player", s.SpellName(124273--[[Heavy Stagger]]), nil, "HARMFUL")
	if name then
		return val2
	end
	return 0
end

function a.incomingDamage2()
	local i
	local ret = 0
	for i = 1, 20, 1 do
		ret = ret + a.IncomingDamage[i]
	end
	return ret
end